#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <netdb.h>

#define MAXLINE 5120

int Socket(int family, int type, int protocol);
int Inet_aton(const char *strptr, struct in_addr *addrptr);
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen);
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen);
ssize_t Send(int sockfd, const void *buf, size_t len, int flags);
ssize_t Recv(int sockfd, void *buf, size_t len, int flags);
int Close(int fd);
struct hostent *Gethostbyname(const char *name);

int main(int argc, char *argv[])
{
	int client_sock;
	struct sockaddr_in server_addr;
	char send_msg[MAXLINE];
	char recv_msg[MAXLINE];

	if (argc != 3) {
		printf("Usage: %s domain port\n", argv[0]);
		exit(1);
	}

	/* 创建socket */
	client_sock = Socket(AF_INET, SOCK_STREAM, 0);

	/* 指定服务器地址 */
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons(atoi(argv[2]));
	server_addr.sin_addr = *(struct in_addr*)Gethostbyname(argv[1])->h_addr_list[0];
	// Inet_aton(argv[2], &server_addr.sin_addr); //127.0.0.1指向本机
	memset(server_addr.sin_zero, 0, sizeof(server_addr.sin_zero)); //零填充

	/* 指定客户端地址 */
	struct sockaddr_in client_addr;
	client_addr.sin_family = AF_INET;
	client_addr.sin_port = htons(23456);
	client_addr.sin_addr.s_addr = htonl(INADDR_ANY); //INADDR_ANY表示本机所有IP地址
	memset(&client_addr.sin_zero, 0, sizeof(client_addr.sin_zero)); //零填充

	/* 绑定socket与地址 */
	Bind(client_sock, (struct sockaddr *)&client_addr, sizeof(client_addr));

	/* 连接服务器 */
	Connect(client_sock, (struct sockaddr *)&server_addr, sizeof(server_addr));

	while (fgets(send_msg, MAXLINE, stdin) != NULL) { // 读取用户的输入
		/* 发送消息 */
		printf("Send: %s", send_msg);
		Send(client_sock, send_msg, strlen(send_msg), 0);

		if (strcasecmp(send_msg, "bye\n") == 0) {	// 如果输入的是 bye 那么就退出
			printf("Bye!\n");
			break;
		}

		/* 接收并显示消息 */
		memset(recv_msg, 0, sizeof(recv_msg)); //接收数组置零
		Recv(client_sock, recv_msg, sizeof(recv_msg), 0);
		printf("Recv: %s", recv_msg);
	}

	/* 关闭socket */
	Close(client_sock);

	return 0;
}

int Socket(int family, int type, int protocol) {
	int n;
	if ((n = socket(family, type, protocol)) < 0) {
		perror("socket error");
		exit(1);
	}
	return n;
}
int Inet_aton(const char *strptr, struct in_addr *addrptr) {
	int n;
	if ((n = inet_aton(strptr, addrptr)) == 0) {
		perror("inet_aton error");
		exit(1);
	}
	return n;
}
int Bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
	int n;
	if ((n = bind(sockfd, addr, addrlen)) < 0) {
		perror("bind error");
		exit(1);
	}
	return n;
}
int Connect(int sockfd, const struct sockaddr *servaddr, socklen_t addrlen) {
	int n;
	if ((n = connect(sockfd, servaddr, addrlen)) < 0) {
		perror("connect error");
		exit(1);
	}
	return n;
}
ssize_t Send(int sockfd, const void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = send(sockfd, buf, len, flags)) < 0) {
		perror("send error");
		exit(1);
	}
	return n;
}
ssize_t Recv(int sockfd, void *buf, size_t len, int flags) {
	ssize_t n;
	if ((n = recv(sockfd, buf, len, flags)) < 0) {
		perror("recv error");
		exit(1);
	}
	return n;
}
int Close(int fd) {
	int n;
	if ((n = close(fd)) < 0) {
		perror("close error");
		exit(1);
	}
	return n;
}
struct hostent *Gethostbyname(const char *name) {
	struct hostent *n;
	if ((n = gethostbyname(name)) == NULL) {
		perror("gethostbyname error");
		exit(1);
	}
	return n;
}